<?php

/**
 * Plugin update notifications
 *
 * Code is based on:
 * https://github.com/jeremyclark13/automatic-theme-plugin-update/blob/master/plugin/test-plugin-update/test-plugin-update.php
 */
if ( ! class_exists( 'E2W_Smallbizdynamic_Plugin_Updates_Gfs' ) ) {

	class E2W_Smallbizdynamic_Plugin_Updates_Gfs {

		const API_URL = 'https://rb.reviewability.com/widget/update';
		const PLUGIN_INFO_URL = 'https://rb.reviewability.com/widget/update/info';
		const DEBUG = 0;

		static private $api_results = array();
		private $plugin_slug = 'testimonials-widget-plugin';
		private $plugin_file;

		function __construct( $plugin_path, $plugin_file ) {
			$this->init( $plugin_path, $plugin_file );
			add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
			add_action( 'admin_init', array( $this, 'admin_init' ) );
		}

		function plugins_loaded() {
			add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check_for_update' ) );
			add_filter( 'plugins_api', array( $this, 'plugin_api_call' ), 10, 3 );
		}

		function admin_init() {
			global $pagenow;
			if ( 'plugin-install.php' === $pagenow && isset( $_GET[ 'tab' ] ) && isset( $_GET[ 'plugin' ] ) ) {
				if ( 'plugin-information' === $_GET[ 'tab' ] && $this->plugin_slug === $_GET[ 'plugin' ] ) {
					header( 'Location: ' . self::PLUGIN_INFO_URL . '?plugin=' . $this->plugin_slug );
					exit;
				}
			}
		}

		function check_for_update( $checked_data ) {
			global $wp_version;
			$action = 'plugin_check';

			$info = $this->get_plugin_info();
			if ( ! $info ) {
				return $checked_data;
			}

			if ( ! isset( self::$api_results[ $this->plugin_slug ] ) ) {
				$send_for_check = array(
					'body' => array(
						'action' => $action,
						'slug' => $this->plugin_slug,
						'version' => $info[ 'Version' ],
						'url' => home_url(),
						'wp' => $wp_version
					),
					'user-agent' => 'WordPress/' . $wp_version . '; ' . home_url()
				);
				$raw_response = wp_remote_post( self::API_URL, $send_for_check );
				self::$api_results[ $this->plugin_slug ][ $action ] = '';

				if ( ! is_wp_error( $raw_response ) && ($raw_response[ 'response' ][ 'code' ] == 200) ) {
					$response = unserialize( $raw_response[ 'body' ] );
				}
				if ( ! empty( $response ) ) {
					$checked_data->response[ $info[ 'LocalPath' ] ] = $response;
					self::$api_results[ $this->plugin_slug ][ $action ] = $response;
				}
			} else if ( ! empty( self::$api_results[ $this->plugin_slug ][ $action ] ) ) {
				$checked_data->response[ $info[ 'LocalPath' ] ] = self::$api_results[ $this->plugin_slug ][ $action ];
			}

			return $checked_data;
		}

		function plugin_api_call( $def, $action, $args ) {
			global $wp_version;

			if ( ! isset( $args->slug ) ) {
				return false;
			}
			if ( $args->slug != $this->plugin_slug ) {
				if ( isset( self::$api_results[ $args->slug ] ) && isset( self::$api_results[ $args->slug ][ $action ] ) ) {
					return self::$api_results[ $args->slug ][ $action ];
				}
				return false;
			}

			$plugin_info = get_site_transient( 'update_plugins' );
			if( ! is_object( $plugin_info ) || ! isset( $plugin_info->checked ) ) {
				return false;
			}
			$info = $this->get_plugin_info();
			$request_string = array(
				'body' => array(
					'action' => $action,
					'slug' => $this->plugin_slug,
					'version' => $plugin_info->checked[ $info[ 'LocalPath' ] ],
					'url' => get_bloginfo( 'url' ),
					'wp' => $wp_version
				),
				'user-agent' => 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' )
			);

			$request = wp_remote_post( self::API_URL, $request_string );

			if ( is_wp_error( $request ) ) {
				$res = new WP_Error( 'plugins_api_failed', __( 'An Unexpected HTTP Error occurred during the API request.</p> <p><a href="?" onclick="document.location.reload(); return false;">Try again</a>' ), $request->get_error_message() );
			} else {
				$res = unserialize( $request[ 'body' ] );

				if ( $res === false ) {
					$res = new WP_Error( 'plugins_api_failed', __( 'An unknown error occurred' ), $request[ 'body' ] );
				}
			}

			self::$api_results[ $args->slug ][ $action ] = $res;
			return $res;
		}

		private function init( $plugin_path, $plugin_file ) {
			if ( self::DEBUG ) {
				set_site_transient( 'update_plugins', null );
			}
			$result = array();
			if ( preg_match( '/\/([^\/]+)\/$/', $plugin_path, $result ) ) {
				$this->plugin_slug = $result[ 1 ];
			}
			$this->plugin_file = $plugin_file;
		}

		private function get_plugin_info() {
			$result = array();
			if ( preg_match( '/\/([^\/]+\/[^\/]+)$/', $this->plugin_file, $result ) ) {
				$plugin_local_path = $result[ 1 ];
				$plugins = get_plugins();
				$result = $plugins[ $plugin_local_path ];
				$result[ 'LocalPath' ] = $plugin_local_path;

				return $result;
			}
			return null;
		}

	}

}
